import java.io.IOException;
import java.io.OutputStream;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;

import util.BinUtil;
import util.GzipUtil;
import util.InByteBuffer;
import util.ProtocolUtil;
import util.SerialUtil;
import util.Str8Buff;
import util.StreamUtils;
import crypt.Cipher;

/**
 * @author Heiner Kcker
 *
 */
public class ProtocolServerBlocked
    extends Thread
{
  /** server port */
  private final int port;

  /** flag to stop this thread */
  private boolean isCanceled = false;

  /** the server socket */
  private ServerSocket serverSocket;

  //Socket inSock = null;

  /** TODO Map fr mehrere Protokoll Handler je nach Operation Code */
  private final ProtocolHandlerInterface protocolHandler;

  static byte[] keyByteArr = "r1U2m3pelsTielzchen".getBytes();

  /**
   * stop this thread
   */
  public void cancel()
  {
    isCanceled = true;
    if (isBlocked)
    {
      // der socket blockiert den programm fluss
      //fieser fix fr das blockieren, sonst fhrt die app niemals runter
      try
      {
        serverSocket.close();
      }
      catch (IOException e)
      {
        //        System.err.println( e.getMessage() );
        //        e.printStackTrace();
      }
    }
  }

  /** Hilfsvariable fr fiesen fix fr shutdown */
  private volatile boolean isBlocked = false;

  /**
   * Constructor
   */
  public ProtocolServerBlocked(final int pPort, ProtocolHandlerInterface pProtocolHandler)
  {
    this.port = pPort;
    if (pProtocolHandler == null)
    {
      throw new NullPointerException();
    }
    this.protocolHandler = pProtocolHandler;
  }

  /* (non-Javadoc)
   * @see java.lang.Thread#run()
   */
  @Override
  public void run()
  {
    while ( !isCanceled)
    {
      // only for develop
      //isCanceld = true;

      Socket inSock = null;
      try
      {

        // TODO umbau auf offen lassen des server socket
        // schliessen zur sicherheit
        //???StreamUtils.close( inSock );

        // -----------------------------------
        // --- receive request form client ---
        // -----------------------------------
        //if ( serverSocket == null || serverSocket.isClosed() )
        {
          StreamUtils.close( serverSocket );
          serverSocket = new ServerSocket( port );

          // accept
          isBlocked = true;
          inSock = serverSocket.accept();
          isBlocked = false;
        }

        Str8Buff reqBuff = ProtocolUtil.readFromSockForLength( inSock );

        final Str8Buff respBuff = handleRequest( reqBuff );

        // get output stream
        final OutputStream outputStream = inSock.getOutputStream();

        // send length of request
        System.out.println( "respBuff.length(): "
            + respBuff.length() );
        outputStream.write( BinUtil.intToByteArr( respBuff.length() ) );

        // send request header and body
        outputStream.write( respBuff.innerByteArr() , 0 , respBuff.length() );
        // ??? outputStream.close();

      }
      catch (BindException be)
      {
        // adress allready in use
        System.exit( 1 );
      }
      catch (Exception e)
      {
        // TODO Log
        System.err.println( e.getMessage() );
        e.printStackTrace();

      }
      finally
      {
        //StreamUtils.close( inSock );
      }
    }
    StreamUtils.close( serverSocket );
  }

  /**
   * @param pReqBuff
   * @return
   * @throws IOException
   */
  private Str8Buff handleRequest(
      Str8Buff pReqBuff )
      throws IOException
  {
    // decrypt request
    // neue instanz des Cipher, damit mehrere Threads diese methode durchlaufen knnen
    final Cipher cipher = new Cipher( keyByteArr );
    //      fr debug off: cipher.decryptIn( reqBuff );

    if (pReqBuff.length() < 9)
    {
      throw new IOException( "invalid message format" );
    }
    final short requestId = BinUtil.byteArrToShort( pReqBuff.sub( 0 , 2 ) );
    System.out.println( "requestId: "
        + requestId );
    final int sessionId = BinUtil.byteArrToInt( pReqBuff.sub( 2 , 6 ) );
    System.out.println( "sessionId: "
        + sessionId );
    final short operationCode = BinUtil.byteArrToShort( pReqBuff.sub( 6 , 8 ) );
    System.out.println( "operationCode: "
        + operationCode );

    // int paramCount = Str8.byteToInt( reqBuff.byteAt( 8 ) );
    // vorzeichen beachten
    int paramCount = pReqBuff.byteAt( 8 );

    System.out.println( "paramCount: "
        + paramCount );
    byte[] paramByteArr = pReqBuff.sub( 9 );
    System.out.println( "paramByteArr: "
        + new String( paramByteArr ) );
    // negativ param count value indicates a compressed body
    final boolean isRequCompressed = paramCount < 0;

    // free memory
    pReqBuff = null;

    if (isRequCompressed)
    {
      paramCount *= -1;
      // decompress response
      paramByteArr = GzipUtil.gunzip( paramByteArr );
    }

    System.out.println( "paramByteArr: "
        + SerialUtil.toArrList( paramByteArr ) );

    final InByteBuffer inByteBuffer = new InByteBuffer( paramByteArr );

    Object[] paramObjArr = SerialUtil.deserializeMulti( inByteBuffer , paramCount );

    Object[] respObjArr;
    boolean compressRespValues = false;
    try
    {
      // call server application code
      respObjArr = this.protocolHandler.excute( //--
          operationCode , //--
          paramObjArr , //--
          requestId , //--
          sessionId );

      // hier kann der app code dem protocol server mitteilen, 
      // ob die daten komprimiert werden sollen
      compressRespValues = this.protocolHandler.isResponseToCompress();

      // test remote exception
      //respObjArr = new Object[]{ new RuntimeException( "test remote exception" ) };
    }
    catch (Exception e)
    {
      // TODO Log
      System.err.println( e.getMessage() );
      e.printStackTrace();

      // send the raised exception to protocol client
      respObjArr = new Object[] { e };
    }

    // free memory
    paramObjArr = null;

    final Str8Buff respBuff = new Str8Buff( 1024 );

    // request id
    respBuff.append( BinUtil.shortToByteArr( requestId ) );

    // session id, TODO byte array zwischenspeichern
    respBuff.append( BinUtil.intToByteArr( sessionId ) );

    // operation code
    respBuff.append( BinUtil.shortToByteArr( operationCode ) );

    // params
    if (respObjArr == null
        || respObjArr.length < 1)
    {
      // keine Parameter
      System.out.println( "keine Parameter" );
      respBuff.append( 0 );
    }
    else
    {
      if (respObjArr.length > Short.MAX_VALUE)
      {
        throw new RuntimeException( "too many return values "
            + respObjArr.length );
      }

      if (compressRespValues)
      {
        // compress params, indicated over highest bit in param count byte
        respBuff.append( (byte) ( respObjArr.length * -1 ) );
      }
      else
      {
        respBuff.append( (byte) respObjArr.length );
      }

      final Str8Buff respValuesBuff = SerialUtil.serializeMulti( respObjArr );
      byte[] respValueByteArr = respValuesBuff.bytes();
      if (compressRespValues)
      {
        respValueByteArr = GzipUtil.gzip( respValueByteArr );
      }
      respBuff.append( respValueByteArr );
    }

    // crypt response
    cipher.reset();
    //        fr debug off: cipher.cryptIn( respBuff );
    return respBuff;
  }

  /**
   * @param args
   */
  public static void main(
      String[] args )
  {
    // TODO Auto-generated method stub

  }

}
