import java.io.IOException;
import java.io.OutputStream;
import java.net.BindException;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.channels.spi.SelectorProvider;
import java.util.HashMap;
import java.util.Iterator;

import util.BinUtil;
import util.GzipUtil;
import util.InByteBuffer;
import util.ProtocolUtil;
import util.SerialUtil;
import util.Str8Buff;
import util.StreamUtils;
import util.ThreadUtil;
import crypt.Cipher;

/**
 * @author Heiner Kcker
 */
public class ProtocolServerNonBlocked
    extends Thread
{
  /** server port */
  private final int port;

  private ServerSocketChannel serverSocketChannel;

  private Selector selector;

  /** flag to stop this thread */
  private boolean isCanceled = false;

  /** TODO Map fr mehrere Protokoll Handler je nach Operation Code */
  private final ProtocolHandlerInterface protocolHandler;

  static byte[] keyByteArr = "r1U2m3pelsTielzchen".getBytes();

  private HashMap<SocketChannel, Session> clientMap = new HashMap<SocketChannel, Session>(1024);

  private static final int CHANNEL_BUFF_SIZE = 4 * 1024;
  //private final ByteBuffer channelByteBuffer = ByteBuffer.allocateDirect( CHANNEL_BUFF_SIZE );
  private final ByteBuffer channelByteBuffer = ByteBuffer.wrap( new byte[ CHANNEL_BUFF_SIZE ] );

  /**
   * stop this thread
   */
  public void cancel()
  {
    isCanceled = true;
      // TODO use utils
      if (serverSocketChannel != null)
      {
        try
        {
          serverSocketChannel.close();
        }
        catch (IOException e)
        {
          //        System.err.println( e.getMessage() );
          //        e.printStackTrace();
        }
      }
  }

  /**
   * Constructor
   */
  public ProtocolServerNonBlocked(//--
      final int pPort, //--
      ProtocolHandlerInterface pProtocolHandler)
  {
    this.port = pPort;
    if (pProtocolHandler == null)
    {
      throw new NullPointerException();
    }
    this.protocolHandler = pProtocolHandler;
  }

  private void startServer()
  throws IOException
  {
    this.serverSocketChannel = ServerSocketChannel.open();
    this.serverSocketChannel.configureBlocking( false );
    final ServerSocket serverSocket = this.serverSocketChannel.socket();
    serverSocket.bind( new InetSocketAddress( this.port ) );
    this.selector = Selector.open();
    serverSocketChannel.register( selector, SelectionKey.OP_ACCEPT );
    // TODO: was einbauen gegen DDOS-Attacke
  }
  
  /* (non-Javadoc)
   * @see java.lang.Thread#run()
   */
  @Override
  public void run()
  {
    try
    {
      startServer();
    }
    catch (Exception e)
    {
      // TODO: Log
      System.err.println( e.getMessage() );
      e.printStackTrace();
      System.exit(1);
    }
    while ( !isCanceled)
    {
      try
      {
        selector.select();
      }
      catch (IOException e)
      {
        // TODO: handle exception
        System.err.println( e.getMessage() );
        e.printStackTrace();
        continue;
      }
      
      Iterator iterator = selector.selectedKeys().iterator();
      while (iterator.hasNext())
      {
        SelectionKey selectionKey = (SelectionKey) iterator.next();
        if ( selectionKey.isWritable() )
        {
          try
          {
            final SocketChannel socketChannel = (SocketChannel) selectionKey.channel();
            final Session session = this.clientMap.get( socketChannel );
            send( socketChannel , session, false);
          }
          catch (IOException e)
          {
            // TODO: log
            System.err.println( e.getMessage() );
            e.printStackTrace();
            
            selectionKey.cancel();
          }
        }
        else if ( selectionKey.isReadable() )
        {
//System.out.println( "selectionKey: " + selectionKey);
          SocketChannel socketChannel = null;
          try
          {
            socketChannel = (SocketChannel) selectionKey.channel();
            final Session session = this.clientMap.get( socketChannel );
            receive( socketChannel , session);
          }
          catch (IOException e)
          {
            // TODO: log
            System.err.println( e.getMessage() );
            e.printStackTrace();
            
            selectionKey.cancel();
            
            StreamUtils.close( socketChannel );
            
            clientMap.remove( socketChannel );
          }
        }
        else if ( selectionKey.isAcceptable() )
        {
          try
          {
            handleAccept();//selectionKey);
          }
          catch (Exception e)
          {
            // TODO: log
            System.err.println( e.getMessage() );
            e.printStackTrace();
            throw new IllegalStateException("unknown key state");
          }
        }
        else
        {
          // TODO log
          System.out.println("unknown key state");
        }
        
      }

    }
    StreamUtils.close( serverSocketChannel );
  }

  /**
   * 
   * @param pChannel
   */
  private void receive( //--
      final SocketChannel pSocketChannel , //--
      final Session pSession )
  throws IOException
  {
    if ( ! pSocketChannel.isConnected() )
    {
      return;
    }

    if (pSession.inBuff == null)
    {
      pSession.inBuff = new Str8Buff( 4 * 1024 );
    }
    final Str8Buff sessionInBuff = pSession.inBuff;

    int numBytesRead = CHANNEL_BUFF_SIZE;

    while (numBytesRead == CHANNEL_BUFF_SIZE )//&& !isCanceled)
    {
      // Create a direct buffer to get bytes from socket.
      // Direct buffers should be long-lived and be reused as much as possible.
      // ByteBuffer socketByteBuffer = ByteBuffer.allocateDirect( channelBuffSize );

    // Clear the buffer and read bytes from socket
    channelByteBuffer.clear();

    numBytesRead = pSocketChannel.read(channelByteBuffer);
    // log.debug("numBytesRead: " + numBytesRead );
    if (numBytesRead < 1)
    {
      // No more bytes can be read from the channel
//      // enforce new logon
//      StreamUtils.close(pSocketChannel);
//      pSocketChannel = null;
//      // return null;
//      break;
      return;
    }
    // To read the bytes, flip the buffer
    channelByteBuffer.flip();

    // Read the bytes from the buffer ...;

//    for (int i = 0; i < numBytesRead; i++)
//    {
//      final byte aByte = channelByteBuffer.get(i);
//      sessionInBuff.append(aByte);
//      // answerBuff.append( aByte );
//    }
    
    sessionInBuff.append( channelByteBuffer.array(), 0 , channelByteBuffer.remaining() );

    }
    
    // Feststellen, ob der Request komplett empfangen wurde
    if ( pSession.isInputComplete() )
    {
      //discard length prefix (wirf weg)
      sessionInBuff.delete(0, 4);
      
      final Str8Buff respBuff = handleRequest( sessionInBuff );
      
      pSession.resetInBuff();

      // insert the length of request
      System.out.println( "respBuff.length(): "
          + respBuff.length() );
      respBuff.insert(0, BinUtil.intToByteArr( respBuff.length() ) );
      
      // send request header and body
      ByteBuffer outByteBuffer = ByteBuffer.wrap( respBuff.bytes() );
      pSession.outByteBuffer = outByteBuffer;
      send( pSocketChannel, pSession , true);
    }
    
  }

  /**
   * @param pSocketChannel
   * @param pSession
   */
  private void send(
      final SocketChannel pSocketChannel ,
      final Session pSession ,
      final boolean pIsFirstCall)
  throws IOException
  {
    final ByteBuffer outByteBuffer = pSession.outByteBuffer;
    if ( outByteBuffer == null )
    {
      return;
    }
    final int toSendLen = outByteBuffer.remaining();
    final int sendedLen = pSocketChannel.write( outByteBuffer );
    if ( toSendLen != sendedLen )
    {
      // client not ready to receive data
      pSocketChannel.register( this.selector, SelectionKey.OP_WRITE );
    }
    else
    {
      // all data sended
      pSession.outByteBuffer = null;
      if ( ! pIsFirstCall )
      {
        pSocketChannel.register( this.selector, SelectionKey.OP_READ );
      }
    }

  }

  /**
   * @param pSelectionKey
   */
  private void handleAccept(
      //SelectionKey pSelectionKey 
      )
    throws IOException
  {
    final SocketChannel socketChannel = this.serverSocketChannel.accept();
    if (socketChannel != null)
    {
      socketChannel.configureBlocking(false);
      socketChannel.register( selector, SelectionKey.OP_READ );
      this.clientMap.put( socketChannel, new Session() );
      System.out.println( "accept count: " +  this.clientMap.size() );
      System.out.println( "socketChannel: " + socketChannel );
      System.out.println( "remote adress: " + ProtocolUtil.getRemoteAdress( socketChannel ));
      
      ThreadUtil.sleepUnhandled( 10000 );
    }
  }

  /**
   * @param pReqBuff
   * @return
   * @throws IOException
   */
  private Str8Buff handleRequest(
      Str8Buff pReqBuff )
      throws IOException
  {
    // decrypt request
    // neue instanz des Cipher, damit mehrere Threads diese methode durchlaufen knnen
    final Cipher cipher = new Cipher( keyByteArr );
    //      fr debug off: cipher.decryptIn( reqBuff );

    if (pReqBuff.length() < 9)
    {
      throw new IOException( "invalid message format" );
    }
    final short requestId = BinUtil.byteArrToShort( pReqBuff.sub( 0 , 2 ) );
    System.out.println( "requestId: "
        + requestId );
    final int sessionId = BinUtil.byteArrToInt( pReqBuff.sub( 2 , 6 ) );
    System.out.println( "sessionId: "
        + sessionId );
    final short operationCode = BinUtil.byteArrToShort( pReqBuff.sub( 6 , 8 ) );
    System.out.println( "operationCode: "
        + operationCode );

    // int paramCount = Str8.byteToInt( reqBuff.byteAt( 8 ) );
    // vorzeichen beachten
    int paramCount = pReqBuff.byteAt( 8 );

    System.out.println( "paramCount: "
        + paramCount );
    byte[] paramByteArr = pReqBuff.sub( 9 );
    System.out.println( "paramByteArr: "
        + new String( paramByteArr ) );
    // negativ param count value indicates a compressed body
    final boolean isRequCompressed = paramCount < 0;

    // free memory
    pReqBuff = null;

    if (isRequCompressed)
    {
      paramCount *= -1;
      // decompress response
      paramByteArr = GzipUtil.gunzip( paramByteArr );
    }

    System.out.println( "paramByteArr: "
        + SerialUtil.toArrList( paramByteArr ) );

    final InByteBuffer inByteBuffer = new InByteBuffer( paramByteArr );

    Object[] paramObjArr = SerialUtil.deserializeMulti( inByteBuffer , paramCount );

    Object[] respObjArr;
    boolean compressRespValues = false;
    try
    {
      // call server application code
      respObjArr = this.protocolHandler.excute( //--
          operationCode , //--
          paramObjArr , //--
          requestId , //--
          sessionId );

      // hier kann der app code dem protocol server mitteilen, 
      // ob die daten komprimiert werden sollen
      compressRespValues = this.protocolHandler.isResponseToCompress();

      // test remote exception
      //respObjArr = new Object[]{ new RuntimeException( "test remote exception" ) };
    }
    catch (Exception e)
    {
      // TODO Log
      System.err.println( e.getMessage() );
      e.printStackTrace();

      // send the raised exception to protocol client
      respObjArr = new Object[] { e };
    }

    // free memory
    paramObjArr = null;

    final Str8Buff respBuff = new Str8Buff( 1024 );

    // request id
    respBuff.append( BinUtil.shortToByteArr( requestId ) );

    // session id, TODO byte array zwischenspeichern
    respBuff.append( BinUtil.intToByteArr( sessionId ) );

    // operation code
    respBuff.append( BinUtil.shortToByteArr( operationCode ) );

    // params
    if (respObjArr == null
        || respObjArr.length < 1)
    {
      // keine Parameter
      System.out.println( "keine Parameter" );
      respBuff.append( 0 );
    }
    else
    {
      if (respObjArr.length > Short.MAX_VALUE)
      {
        throw new RuntimeException( "too many return values "
            + respObjArr.length );
      }

      if (compressRespValues)
      {
        // compress params, indicated over highest bit in param count byte
        respBuff.append( (byte) ( respObjArr.length * -1 ) );
      }
      else
      {
        respBuff.append( (byte) respObjArr.length );
      }

      final Str8Buff respValuesBuff = SerialUtil.serializeMulti( respObjArr );
      byte[] respValueByteArr = respValuesBuff.bytes();
      if (compressRespValues)
      {
        respValueByteArr = GzipUtil.gzip( respValueByteArr );
      }
      respBuff.append( respValueByteArr );
    }

    // crypt response
    cipher.reset();
    //        fr debug off: cipher.cryptIn( respBuff );
    return respBuff;
  }

  /**
   * @param args
   */
  public static void main(
      String[] args )
  {
    // TODO Auto-generated method stub

  }

}
