package crypt;

import java.io.IOException;
import java.io.OutputStream;

/**
 * OutputStream zur Verschlsselung.
 * Es wird eine einfache symmetrische Verschlsselung benutzt.
 * Er dient lediglich zum Erschweren des Reeingenering des Clients,
 * die Sicherheit der Verschlsselung ist weniger wichtig.
 * 
 * @author Heiner Kcker
 */
public class CryptStream
    extends OutputStream
{
  /** dekorierter Stream */
  private final OutputStream decoratedOutputStream;

  private final Cipher cipher;

  /**
   * Constructor
   */
  public CryptStream( //--
      final OutputStream pOutputStream, //--
      final byte[] pCryptKeyByteArr)
  {
    super();
    this.decoratedOutputStream = pOutputStream;

    this.cipher = new Cipher( pCryptKeyByteArr );
  }

  /* (non-Javadoc)
   * @see java.io.OutputStream#write(int)
   */
  @Override
  public void write(
      int pByte )
      throws IOException
  {
    int cryptByte = this.cipher.cryptOne( pByte );

//System.out.println( "write: " + cryptByte );
    decoratedOutputStream.write( cryptByte );
    
    // als Hex-Zahl
    //byte[] hexByteArr = byte2Hex( cryptByte );
    //decoratedOutputStream.write( hexByteArr );
  }

  /**
   * Umwandlung byte (als int) in
   * einen zweistelligen Hexadezimal-String
   * 
   * @param pCryptByte byte als int
   * @return zweistelliger Hexadezimal-String
   */
  private static byte[] byte2Hex(
      final int pCryptByte )
  {
    String hexStr = Integer.toHexString( pCryptByte );
    if (hexStr.length() > 1)
    {
      return hexStr.getBytes();
    }
    return ( "0" + hexStr ).getBytes();
  }

}
