package crypt;

import java.io.IOException;
import java.io.InputStream;

/**
 * @author Heiner Kcker
 */
public class EncryptStream
    extends InputStream
{

  /** dekorierter Stream */
  private final InputStream decoratedInputStream;

  private final Cipher cipher;

  /**
   * Constructor
   */
  public EncryptStream( //--
      final InputStream pInputStream, //--
      final byte[] pCryptKeyByteArr)
  {
    super();
    this.decoratedInputStream = pInputStream;

    if (pCryptKeyByteArr == null
        || pCryptKeyByteArr.length < 1)
    {
      throw new IllegalArgumentException();
    }

    this.cipher = new Cipher( pCryptKeyByteArr );
  }

  /* (non-Javadoc)
   * @see java.io.InputStream#read()
   */
  @Override
  public int read()
      throws IOException
  {
    int retByte = decoratedInputStream.read();
//System.out.println( "read: " + retByte );
    if (retByte == -1)
    {
      return -1;
    }
    
    // als Hex-Zahl
//    final int inByte1 = decoratedInputStream.read();
//    if (inByte1 == -1)
//    {
//      return -1;
//    }
//    int retByte = hex2byte( inByte1 , decoratedInputStream.read() );
//
    
    final int decryptByte = this.cipher.decryptOne( retByte );
//System.out.println( "decryptByte: " + decryptByte );
    return decryptByte;

  }

  /**
   * Umwandlung eines zweistelligen Hexadezimal-Code
   * (char als int von einem {@link InputStream}) 
   * in ein byte (als int)
   * 
   * @param pByte1 Hexadezimal-Code erste Stelle
   * @param pByte2 Hexadezimal-Code zweite Stelle
   * @return byte (als int)
   */
  private static int hex2byte(
      final int pByte1 , //
      final int pByte2 )
  {
    char c1 = (char) pByte1;
    char c2 = (char) pByte2;

    String str = String.valueOf( c1 )
        + String.valueOf( c2 );

    return Integer.valueOf( str , 16 );
  }

}
