package util;

/**
 * Util class to convert java
 * integer values to byte array
 * and vice versa.
 * 
 * @author Heiner Kcker
 */
public class BinUtil
{

  /**
   * Constructor
   */
  private BinUtil()
  {
    super();
    // TODO Auto-generated constructor stub
  }

  /**
   * Convert an short value (16 bit)
   * in a byte-Array with highest
   * significant Byte first.
   * 
   * @param pShort
   * @return
   */
  public static byte[] shortToByteArr(
      short pShort )
  {
    int tShort = Str8.shortToInt( pShort );
    final byte[] retByteArr = new byte[2];
    retByteArr[1] = (byte) ( tShort & 0xff );
    tShort = tShort >>> 8;
    retByteArr[0] = (byte) tShort;
    return retByteArr;
  }

  /**
   * Deconvert the given byte array to
   * short value.
   * 
   * @param pByteArr
   * @return
   */
  public static int shortByteArrToInt(
      final byte[] pByteArr )
  {
    int retInt;
    retInt = pByteArr[0] & 0xff;
    retInt = retInt << 8;
    retInt += pByteArr[1] & 0xff;
    return retInt;
  }

  /**
   * Convert an int value (32 bit)
   * in a byte-Array with highest
   * significant Byte first.
   * 
   * @param pInt
   * @return
   */
  public static byte[] intToByteArr(
      int pInt )
  {
    final byte[] retByteArr = new byte[4];
    retByteArr[3] = (byte) ( pInt & 0xff );
    pInt = pInt >>> 8;
    retByteArr[2] = (byte) ( pInt & 0xff );
    pInt = pInt >>> 8;
    retByteArr[1] = (byte) ( pInt & 0xff );
    pInt = pInt >>> 8;
    retByteArr[0] = (byte) pInt;
    return retByteArr;
  }

  /**
   * Deconvert the given byte array to
   * short value.
   * 
   * @param pByteArr
   * @return
   */
  public static short byteArrToShort(
      final byte[] pByteArr )
  {
    int retInt;
    retInt = pByteArr[0] & 0xff;
    retInt = retInt << 8;
    retInt += pByteArr[1] & 0xff;
    return (short) retInt;
  }

  /**
   * Deconvert the given byte array to
   * int value.
   * 
   * @param pByteArr
   * @return
   */
  public static int byteArrToInt(
      final byte[] pByteArr )
  {
    int retInt;
    retInt = pByteArr[0] & 0xff;
    retInt = retInt << 8;
    retInt += pByteArr[1] & 0xff;
    retInt = retInt << 8;
    retInt += pByteArr[2] & 0xff;
    retInt = retInt << 8;
    retInt += pByteArr[3] & 0xff;
    return retInt;
  }

  /**
   * Deconvert the given {@link InByteBuffer}
   * to short value.
   * 
   * @param pInByteBuffer
   * @return
   */
  public static short readShort(
      final InByteBuffer pInByteBuffer )
  {
    int retInt;
    retInt = pInByteBuffer.read() & 0xff;
    retInt = retInt << 8;
    retInt += pInByteBuffer.read() & 0xff;
    return (short) retInt;
  }

  /**
   * Deconvert the given {@link InByteBuffer}
   * to int value.
   * 
   * @param pInByteBuffer
   * @return
   */
  public static int readInt(
      final InByteBuffer pInByteBuffer )
  {
    int retInt;
    retInt = pInByteBuffer.read() & 0xff;
    retInt = retInt << 8;
    retInt += pInByteBuffer.read() & 0xff;
    retInt = retInt << 8;
    retInt += pInByteBuffer.read() & 0xff;
    retInt = retInt << 8;
    retInt += pInByteBuffer.read() & 0xff;
    return retInt;
  }

  /**
   * Convert an long value (64 bit)
   * in a byte-Array with highest
   * significant Byte first.
   * 
   * @param pLong
   * @return
   */
  public static byte[] longToByteArr(
      long pLong )
  {
    final byte[] retByteArr = new byte[8];
    retByteArr[7] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[6] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[5] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[4] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[3] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[2] = (byte) ( pLong & 0xff );
    pLong = pLong >>> 8;
    retByteArr[1] = (byte) pLong;
    pLong = pLong >>> 8;
    retByteArr[0] = (byte) pLong;
    return retByteArr;
  }

  /**
   * Deconvert the given byte array to
   * long value.
   * 
   * @param pByteArr
   * @return
   */
  public static long byteArrToLong(
      final byte[] pByteArr )
  {
    long retLong;
    retLong = pByteArr[0] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[1] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[2] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[3] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[4] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[5] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[6] & 0xff;
    retLong = retLong << 8;
    retLong += pByteArr[7] & 0xff;
    return retLong;
  }

  /**
   * Deconvert the given byte array to
   * long value.
   * 
   * @param pInByteBuffer
   * @return
   */
  public static long readLong(
      final InByteBuffer pInByteBuffer )
  {
    long retLong;
    retLong = pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    retLong = retLong << 8;
    retLong += pInByteBuffer.read() & 0xff;
    return retLong;
  }

  /**
   * Check the specified Number
   * for short (16 Bit) value range.
   * 
   * @param pNumber
   * @return
   */
  public static boolean isShort(
      final Number pNumber )
  {
    if (pNumber instanceof Short
        || pNumber instanceof Byte)
    {
      return true;
    }
    if (pNumber instanceof Integer
        || pNumber instanceof Long)
    {
      long longVal = pNumber.longValue();
      return longVal >= Short.MIN_VALUE
          && longVal <= Short.MAX_VALUE;
    }

    return false;
  }

  /**
   * @param pByteArr
   * @return
   */
  public static float byteArrToFloat(
      final byte[] pByteArr )
  {
    final int floatBinaryInt = byteArrToInt( pByteArr );

    final float retFloat = Float.intBitsToFloat( floatBinaryInt );
    return retFloat;
  }

  /**
   * @param pFloat
   * @return
   */
  public static byte[] floatToByteArr(
      final float pFloat )
  {
    final int floatBinaryInt = Float.floatToIntBits( pFloat );

    final byte[] intByteArr = intToByteArr( floatBinaryInt );
    return intByteArr;
  }

  /**
   * @param lpByteArr
   * @return
   */
  public static float readFloat(
      final InByteBuffer pInByteBuffer )
  {
    final int floatBinaryInt = byteArrToInt( pInByteBuffer.read( 4 ) );

    final float retFloat = Float.intBitsToFloat( floatBinaryInt );
    return retFloat;
  }

  /**
   * @param pByteArr
   * @return
   */
  public static double byteArrToDouble(
      final byte[] pByteArr )
  {
    final long doubleBinaryLong = byteArrToLong( pByteArr );

    final double retDouble = Double.longBitsToDouble( doubleBinaryLong );
    return retDouble;
  }

  /**
   * @param pDouble
   * @return
   */
  public static byte[] doubleToByteArr(
      final double pDouble )
  {
    final long doubleBinaryLong = Double.doubleToLongBits( pDouble );

    final byte[] longByteArr = longToByteArr( doubleBinaryLong );
    return longByteArr;
  }

  /**
   * @param lpByteArr
   * @return
   */
  public static double readDouble(
      final InByteBuffer pInByteBuffer )
  {
    final long doubleBinaryLong = byteArrToLong( pInByteBuffer.read( 8 ) );

    final double retDouble = Double.longBitsToDouble( doubleBinaryLong );
    return retDouble;
  }

  /**
   * @param args
   */
  public static void main(
      String[] args )
  {
    if (false)
    {
      // Test intToByteArr und byteArrToInt
      for (int i = Integer.MIN_VALUE; i < Integer.MAX_VALUE; i++ )
      {
        byte[] intByteArr = BinUtil.intToByteArr( i );
        int deserialInt = BinUtil.byteArrToInt( intByteArr );
        if (deserialInt != i)
        {
          throw new RuntimeException( ""
              + i );
        }
      }
    }
    System.out.println( "ok" );
  }

}
