package util;

/**
 * Hilfsmethoden zum Vergleich von Objekten . <br/>
 * 
 * <br/> <b>Author:</b> <a href="http://www.heinerkuecker.de"
 * target="_blank">Heiner Kcker</a> <br/>
 * 
 * @author Heiner Kcker
 */
public final class CompareUtil
{

  /**
   * compareTo-Methode mit Abfangung null
   */
  public static final int compareTo(Comparable comPa1, Comparable comPa2)
  {
    // Log.debug( "CompareUtil#compareTo >" + comPa1 + "< with >" + comPa2 + "<"
    // );
    if (comPa1 == null && comPa2 == null)
    {
      // wenn comPa1 und comPa1 null ist, dann gilt gleich
      return 0;
    }
    else if (comPa1 == null)
    {
      // wenn comPa1 null ist, dann gilt kleiner als
      return -1;
    }
    else if (comPa2 == null)
    {
      // wenn comPa2 null ist, dann gilt groesser als
      return 1;
    }
    else
    {
      try
      {
        return comPa1.compareTo(comPa2);
      }
      catch (ClassCastException c)
      {
        String classStr1 = comPa1 == null ? "null" : comPa1.getClass().getName();
        String classStr2 = comPa2 == null ? "null" : comPa2.getClass().getName();
        throw new ClassCastException("compare " + classStr1 + " with " + classStr2);
      }
    }// end if
  }// end method compareTo

  // public static final boolean isEqual(Comparable comPa1, Comparable comPa2) {
  // return (compareTo(comPa1, comPa2) == 0);
  // }// end method equals
  //
  // public static final boolean notEquals(Comparable comPa1, Comparable comPa2)
  // {
  // return !(compareTo(comPa1, comPa2) == 0);
  // }// end method notEquals
  //

  /**
   * equal-methode mit Abfangung null
   */
  public static final boolean isEqual(Object pObj1, Object pObj2)
  {
    if (pObj1 == null && pObj2 == null)
    {
      // wenn comPa1 und comPa1 null ist, dann gilt gleich
      return true;
    }
    else if (pObj1 == null)
    {
      // wenn comPa1 null ist, dann gilt kleiner als
      return false;
    }
    else if (pObj2 == null)
    {
      // wenn comPa2 null ist, dann gilt groesser als
      return false;
    }
    else
    {
      return pObj1.equals(pObj2);
    }// end if
  }

  public static final boolean notEquals(Object pObj1, Object pObj2)
  {
    return !isEqual(pObj1, pObj2);
  }

  /**
   * kleiner als.
   * 
   * @param comPa1
   * @param comPa2
   * @return
   */
  public static final boolean lesser(Comparable comPa1, Comparable comPa2)
  {
    return (compareTo(comPa1, comPa2) < 0);
  }// end method less

  /**
   * kleiner oder gleich.
   * 
   * @param comPa1
   * @param comPa2
   * @return
   */
  public static final boolean lesserEqual(Comparable comPa1, Comparable comPa2)
  {
    return (compareTo(comPa1, comPa2) <= 0);
  }// end method lessEqual

  /**
   * grsser als.
   * 
   * @param comPa1
   * @param comPa2
   * @return
   */
  public static final boolean greater(Comparable comPa1, Comparable comPa2)
  {
    return (compareTo(comPa1, comPa2) > 0);
  }

  /**
   * grsser oder gleich.
   * 
   * @param comPa1
   * @param comPa2
   * @return
   */
  public static final boolean greaterEqual(Comparable comPa1, Comparable comPa2)
  {
    return (compareTo(comPa1, comPa2) >= 0);
  }

  /**
   * Test auf Einhaltung eines Bereiches.
   * 
   * @param comIn
   * @param comFrom
   * @param comTo
   * @return
   */
  public static final boolean inRange(Comparable comIn, Comparable comFrom, Comparable comTo)
  {
    return (greaterEqual(comIn, comFrom) && lesserEqual(comIn, comTo));
  }

}
