package util;

/**
 * Replace (holder) class for exception to 
 * transport a Exception over the Protocol.
 * 
 * @author Heiner Kcker
 * 
 * TODO nested exceptions
 */
public class ExceptionHolder
{
  public final String className;
  
  public final String message;
  
  public final String stacktrace;

  /** chain of nested exceptions */
  public final ExceptionHolder causeExceptionHolder;
  
  /**
   * Constructor with exception paramter.
   */
  public ExceptionHolder(
      final Throwable pThrowable)
  {
    this.className = pThrowable.getClass().getName();
    this.message = pThrowable.getMessage();
    this.stacktrace = ExceptionUtil.getStackTrace( pThrowable );
    final Throwable causeThrowable = pThrowable.getCause();
    if ( causeThrowable != null )
    {
      // build an recursiv chain of nested exceptions
      this.causeExceptionHolder = new ExceptionHolder( causeThrowable );
    }
    else
    {
      this.causeExceptionHolder = null;
    }
  }

  /**
   * Constructor with string paramter.
   */
  public ExceptionHolder(
      final String pClassName,
      final String pMessage,
      final String pStacktrace,
      final ExceptionHolder pCauseExceptionHolder)
  {
    this.className = pClassName;
    this.message = pMessage;
    this.stacktrace = pStacktrace;
    this.causeExceptionHolder = pCauseExceptionHolder;
  }

  /**
   * dump out the exception.
   */
  @Override
  public String toString()
  {
    return this.className + ": " + this.message + '\n' + this.stacktrace +
    ( this.causeExceptionHolder == null ? "" : "\ncaused by nested exception: " + this.causeExceptionHolder);
  }

  /* (non-Javadoc)
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(
      Object pObj )
  {
    if ( pObj == null )
    {
      return false;
    }
    if ( ! ( pObj instanceof ExceptionHolder ))
    {
      return false;
    }
    final  ExceptionHolder other = (ExceptionHolder) pObj;
    return CompareUtil.isEqual( this.className , other.className )
      && CompareUtil.isEqual( this.message , other.message )
      && CompareUtil.isEqual( this.stacktrace , other.stacktrace )
      && CompareUtil.isEqual( this.causeExceptionHolder , other.causeExceptionHolder );
  }

}
