package util;

import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;
import java.nio.channels.SocketChannel;
import java.util.Arrays;

/**
 * @author Heiner Kcker
 */
public class ProtocolUtil
{

  /**
   * Constructor
   */
  private ProtocolUtil()
  {
    super();
    // TODO Auto-generated constructor stub
  }

  /** maximum size of received data */
  public static final int MAX_PROTOCOL_SIZE = 16 * 1024 * 1024;
  
  /**
   * Read data from socket input stream
   * by according the the length prefix.
   * 
   * @param pSocket TODO
   * @return
   * @throws IOException
   */
  public static Str8Buff readFromSockForLength(Socket pSocket)
      throws IOException
  {
    final InputStream inputStream = pSocket.getInputStream();
    final Str8Buff buff = new Str8Buff( 1024 );
    int readLength = MAX_PROTOCOL_SIZE;
    boolean isLengthComputed = false;
    int inByte;
    //while( (inByte = inputStream.read()) != -1 )
    //while(true)
    while( pSocket.isConnected() )
    {
      inByte = inputStream.read();
      if ( inByte != -1 )
      {
//System.out.println("inByte: " + inByte );        
        buff.append( inByte );
        if ( ! isLengthComputed && buff.length() == 4 )
        {
          // read lentgh prefix
          readLength = BinUtil.byteArrToInt( buff.innerByteArr() );
System.out.println( "readLength: " + readLength );
          
          isLengthComputed = true;

          if ( readLength > MAX_PROTOCOL_SIZE )
          {
            throw new IOException( "to big data block: " + readLength );
          }
          if ( readLength < 0 )
          {
            throw new IOException( "data block size is lesser than zero: " + readLength );
          }
          
          // discard length prefix (wirf weg)
          buff.reset();
        }
        if ( buff.length() >= readLength )
        {
          // response end reached
System.out.println( "break at respBuff.length(): " + buff.length());
          break;
        }
      }
      else
      {
        // -1 end of stream
        //System.out.println( "read -1" );
        ThreadUtil.sleepUnhandled( 100 );
      }
    }
    System.out.println( "readFromSockForLength: '" + buff + "'" );
    System.out.println( "readFromSockForLength: " + SerialUtil.toArrList( buff.bytes() ) );
    return buff;
  }

  /**
   * Ermittle die Remote-Adresse eines SocketChannel
   * (der Cient im Server-Fall).
   * 
   * @param pSocketChannel
   * @return
   */
  public static long getRemoteAdress( //--
      final SocketChannel pSocketChannel)
  {
//    if (pSocketChannel == null)
//    {
//      return Long.MIN_VALUE;
//    }
    final String scktChnlStr = pSocketChannel.toString();
    return ipStr2long( scktChnlStr, scktChnlStr.lastIndexOf( '/' ) + 1 );
  }


  /**
   * Ermittel aus dem bergebenen String
   * ab der bergebenen Position eine
   * IP-Adresse als long-Wert
   * 
   * @param pIpStr
   * @param pPos
   * @return
   */
  public static long ipStr2long(
      final String pIpStr ,
      final int pPos )
  {
    long ipValLong = 0;
    int pos = pPos;
    runLoop : // label
    while ( pos < pIpStr.length() )
    {
      char digitChr = pIpStr.charAt( pos );
      switch (digitChr)
      {
        case '0' :
        case '1' :
        case '2' :
        case '3' :
        case '4' :
        case '5' :
        case '6' :
        case '7' :
        case '8' :
        case '9' :
          // add the digit value
          ipValLong = ( ipValLong * 10 ) + ( digitChr - 48 );
          break;
        case '.' :
          // move to next byte
          ipValLong *= 256;
          break;
        default:
          // break loop to label
          break runLoop;
      }
      pos++;
      //System.out.println( digitChr );
      //System.out.println( ipValLong );
    }
    return ipValLong;
  }
  
}
